// Class  : GeneralServer
// Author : Russ Ethington
// Version: 1/28/96 for Java 1.0

import java.lang.*;
import java.net.*;
import java.io.*;  

class GeneralServer implements Runnable
{
   private ConnectionHandler owner;

   private ServerSocket      server = null;
   private Socket            socket = null;
   private Thread            life = null;
   
   private DataInputStream   in = null;
   private PrintStream       out = null;
   
   private boolean           runState = false;
   private boolean           connectState = false;
   final int                 defaultPort = 4646;
   
   public GeneralServer(ConnectionHandler theOwner)
   {
      owner = theOwner;
   }

   public void start()
   {
      start(defaultPort);
   }

   public void start(int thePort)
   {
      owner.debugLog("GeneralServer:start", this);
      runState = true;
      if (life == null)
      {
         life = new Thread(this);
         life.setPriority(Thread.MAX_PRIORITY);
      }
      try
         {server = new ServerSocket(thePort);}
      catch (IOException e)
      {
         owner.changeStatus("Could not create new server...", this);
         return;
      }            
      life.start();
   }

   public void stop()
   {
      owner.debugLog("GeneralServer:stop", this);
      if (life != null) 
         life.stop();
      shutdown();
   }

   public boolean isRunning()
   {
      owner.debugLog("GeneralServer:isRunning", this);
      return runState;
   }

   public boolean isConnected()
   {
      owner.debugLog("GeneralServer:isConnected", this);
      return connectState;
   }

   public void run()
   {
      owner.debugLog("GeneralServer:run", this);
      while (true)
      {
         try
            {socket = server.accept();}
         catch (IOException e)
         {
            owner.changeStatus("Could not accept incoming calls...", this);
            shutdown();
            return;
         }

         try
         {
            in = new DataInputStream(
                     new BufferedInputStream(socket.getInputStream()));
            out = new PrintStream(
                      new BufferedOutputStream(socket.getOutputStream()), true);  
         }
         catch (IOException e)
         {
            owner.changeStatus("Could not create data streams...", this);
            shutdown();
            return;
         }
         owner.connectionUp("Connected.", this);
         connectState = true;
         owner.handleSession(in, out, this);
         connectState = false;
         owner.connectionDown("Disconnected.", this);
         try
         {
            in.close();
            out.close();
         }  catch (IOException e) { }
      }
   }

   public void shutdown()
   {
      owner.debugLog("GeneralServer:shutdown", this);
      life = null;
      runState = false;
      if (connectState)
      {
         try
         {
            in.close();
            out.close();
         }  catch (IOException e) { }
         connectState = false;
         if (socket != null)
         {
            try {socket.close();}
            catch (Exception e) { }
         }
      }
      socket = null;
      server = null;
      
      in = null;
      out = null;
   }
}
